# -*- coding: utf-8 -*-

from Acquisition import aq_parent
from collective.taxonomy.interfaces import ITaxonomy
from imio.smartweb.common.utils import is_log_active
from imio.smartweb.core.config import WCA_URL
from imio.smartweb.core.contents import IFolder
from more_itertools import chunked
from plone import api
from plone.app.multilingual.interfaces import ILanguageRootFolder
from plone.dexterity.interfaces import IDexterityContent
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.defaultpage import get_default_page
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from Products.CMFPlone.utils import base_hasattr
from urllib.parse import urlparse, urlunparse
from zope.component import getSiteManager
from zope.component import getUtility
from zope.component import queryMultiAdapter

import base64
import hashlib
import json
import logging
import os
import re
import requests

logger = logging.getLogger("imio.smartweb.core")


def get_category(context):
    if not base_hasattr(context, "category_name"):
        return
    field_name = "taxonomy_{}".format(context.category_name)
    taxonomy_name = "collective.taxonomy.{}".format(context.category_name)
    term = getattr(context, field_name)
    if not term:
        return
    current_lang = api.portal.get_current_language()[:2]
    sm = getSiteManager()
    utility = sm.queryUtility(ITaxonomy, name=taxonomy_name)
    value = utility.translate(
        term,
        context=context,
        target_language=current_lang,
    )
    return value


def get_categories():
    sm = getSiteManager()
    return sm.queryUtility(ITaxonomy, name="collective.taxonomy.page_category")


def concat_voca_term(term1, term2):
    return "{0}-{1}".format(term1, term2)


def concat_voca_title(title1, title2):
    return "{0} - {1}".format(title1, title2)


def get_json(url, auth=None, timeout=5):
    language = api.portal.get_current_language()
    headers = {"Accept": "application/json", "Cookie": f"I18N_LANGUAGE={language}"}
    if auth is not None:
        headers["Authorization"] = auth
    try:
        response = requests.get(url, headers=headers, timeout=timeout)
    except requests.exceptions.Timeout:
        logger.warning(f"Timeout raised for requests : {url}")
        return None
    except Exception:
        return None
    if response.status_code != 200:
        return None
    if response.text:
        return json.loads(response.text)


def get_wca_token(client_id, client_secret):
    username = os.environ.get("RESTAPI_USER_USERNAME")
    password = os.environ.get("RESTAPI_USER_PASSWORD")

    payload = {
        "grant_type": "password",
        "client_id": client_id,
        "client_secret": client_secret,
        "username": username,
        "password": password,
        "scope": ["openid"],
    }
    if not client_id or not client_secret:
        return (username, password)
    headers = {
        "Content-Type": "application/x-www-form-urlencoded",
    }
    response = requests.post(WCA_URL, headers=headers, data=payload)
    id_token = response.json().get("id_token")
    if is_log_active():
        logger.info(f"WCA token retrieved : {id_token}")
    return "Bearer {0}".format(id_token)


def hash_md5(text):
    return hashlib.md5(text.encode()).hexdigest()


def safe_html(html):
    if not html:
        return
    transforms = api.portal.get_tool("portal_transforms")
    data = transforms.convertTo(target_mimetype="text/x-html-safe", orig=html)
    output = data.getData()
    return output


def batch_results(iterable, batch_size):
    return list(chunked(iterable, batch_size, strict=False))


def reindexParent(obj, event=None):
    parent = aq_parent(obj)
    if parent is not None:
        # in some cases (ex: relation breaking), we do not get the object in
        # its acquisition chain
        parent.reindexObject()


def get_default_content_id(obj):
    if IPloneSiteRoot.providedBy(obj) or ILanguageRootFolder.providedBy(obj):
        # Plone / LRF default page
        item_id = get_default_page(obj)
        return item_id and item_id or ""
    elif IFolder.providedBy(obj):
        # Our folder default page
        item = obj.get_default_item()
        return item and item.getId or ""


def get_scale_url(context, request, fieldname, scale_name, orientation=""):
    if orientation:
        m = re.match(r"(portrait|paysage|carre)_(\w+)", scale_name)
        if m:
            # remove existing orientation (if any) from scale name
            scale_name = m.group(2)
    scale_name = "_".join(filter(None, [orientation, scale_name]))
    if IDexterityContent.providedBy(context):
        # get scale url on an object
        if not scale_name:
            # return the full image
            modified_hash = hash_md5(context.ModificationDate())
            return f"{context.absolute_url()}/@@images/{fieldname}/?cache_key={modified_hash}"
        images_view = queryMultiAdapter((context, request), name="images")
        if images_view is None:
            return ""
        scale = images_view.scale(fieldname, scale_name)
        if scale is None:
            return ""
        return scale.url
    else:
        # get scale url on a brain
        # In this case, we need a modification hash to handle croppings, because
        # catalog does not handle them correctly.
        # See https://github.com/collective/plone.app.imagecropping/issues/129
        brain = context
        if fieldname == "image" and not brain.has_leadimage:
            return ""
        modification_date = brain.ModificationDate
        if callable(modification_date):
            # brain in content listing for example
            modification_date = modification_date()
        modified_hash = hash_md5(modification_date)
        url = f"{brain.getURL()}/@@images/{fieldname}/{scale_name}?cache_key={modified_hash}"
        return url


def get_plausible_vars():
    env_plausible_url = os.getenv("SMARTWEB_PLAUSIBLE_URL", "")
    env_plausible_site = os.getenv("SMARTWEB_PLAUSIBLE_SITE", "")
    env_plausible_token = os.getenv("SMARTWEB_PLAUSIBLE_TOKEN", "")

    plausible_url = (
        env_plausible_url
        if (env_plausible_url and env_plausible_url != "")
        else api.portal.get_registry_record("smartweb.plausible_url")
    )
    plausible_site = (
        env_plausible_site
        if (env_plausible_site and env_plausible_site != "")
        else api.portal.get_registry_record("smartweb.plausible_site")
    )
    plausible_token = (
        env_plausible_token
        if (env_plausible_token and env_plausible_token != "")
        else api.portal.get_registry_record("smartweb.plausible_token")
    )
    if all([plausible_site, plausible_url, plausible_token]):
        plausible_vars = {
            "plausible_url": plausible_url,
            "plausible_site": plausible_site,
            "plausible_token": plausible_token,
        }
        return plausible_vars
    else:
        return None


def get_value_from_registry(key):
    val = api.portal.get_registry_record(key)
    return val


def get_iadeliberation_url_from_registry():
    iadeliberation_url = api.portal.get_registry_record("smartweb.iadeliberations_url")
    return iadeliberation_url


def get_iadeliberation_institution_from_registry():
    iadeliberation_institution = api.portal.get_registry_record(
        "smartweb.iadeliberations_institution"
    )
    return iadeliberation_institution


def get_iadeliberation_json(url):
    iadeliberation_user = api.portal.get_registry_record(
        "smartweb.iadeliberations_api_username"
    )
    iadeliberation_pwd = api.portal.get_registry_record(
        "smartweb.iadeliberation_api_password"
    )
    usrPass = f"{iadeliberation_user}:{iadeliberation_pwd}".encode("utf-8")
    b64Val = base64.b64encode(usrPass)
    json = get_json(url, auth=f"Basic {b64Val.decode('utf-8')}", timeout=20)

    return json


def get_iaideabox_json(url):
    user = api.portal.get_registry_record("smartweb.iaideabox_api_username")
    pwd = api.portal.get_registry_record("smartweb.iaideabox_api_password")
    usrPass = f"{user}:{pwd}".encode("utf-8")
    b64Val = base64.b64encode(usrPass)
    json = get_json(url, auth=f"Basic {b64Val.decode('utf-8')}", timeout=20)
    return json


def get_basic_auth_json(url, user, pwd):
    usrPass = f"{user}:{pwd}".encode("utf-8")
    b64Val = base64.b64encode(usrPass)
    json = get_json(url, auth=f"Basic {b64Val.decode('utf-8')}", timeout=20)
    return json


def is_valid_url(url: str) -> bool:
    try:
        parsed = urlparse(url)
        return all([parsed.scheme in ("http", "https"), parsed.netloc])
    except Exception:
        return False


def get_ts_api_url(service="wcs"):
    url_ts = get_value_from_registry("smartweb.url_ts")
    if url_ts is None:
        return None
    if not is_valid_url(url_ts):
        return None
    parsed_url = urlparse(url_ts)
    if "wcs" in service:
        new_netloc = parsed_url.netloc.replace(
            ".guichet-citoyen.be", "-formulaires.guichet-citoyen.be"
        )
    api_url = urlunparse(parsed_url._replace(netloc=new_netloc, path="/api"))
    return api_url


def remove_cache_key(json_data: dict) -> dict:
    pattern = re.compile(r"&cache_key=[a-f0-9]{32}")
    if json_data is None:
        return json_data
    if "@id" in json_data and isinstance(json_data["@id"], str):
        json_data["@id"] = pattern.sub("", json_data["@id"])

    # Nested keys inside 'batching'
    if "batching" in json_data and isinstance(json_data["batching"], dict):
        for key in ["@id", "first", "last", "next"]:
            if key in json_data["batching"] and isinstance(
                json_data["batching"][key], str
            ):
                json_data["batching"][key] = pattern.sub("", json_data["batching"][key])

    return json_data


def populate_procedure_button_text():
    """Populate the procedure_button_text in the control panel registry."""
    registry = getUtility(IRegistry)
    current = registry.get("smartweb.procedure_button_text", None)
    # if not current:
    #     logger.info("Populating smartweb.procedure_button_text in registry")
    registry["smartweb.procedure_button_text"] = [
        {
            "label_id": "label-1",
            "label_fr": "Effectuer cette démarche en ligne",
            "label_nl": "Voltooi deze procedure online",
            "label_de": "Diese Prozedur online abschließen",
            "label_en": "Complete this procedure online",
        },
        {
            "label_id": "label-2",
            "label_fr": "Postuler",
            "label_nl": "Solliciteren",
            "label_de": "Bewerben",
            "label_en": "Apply",
        },
    ]



from imio.smartweb.common.browser.forms import CustomAddForm
from imio.smartweb.common.config import APPLICATION_ID
from imio.smartweb.common.config import PROJECT_ID
from plone.dexterity.browser.add import DefaultAddView

from z3c.form.interfaces import HIDDEN_MODE, DISPLAY_MODE
from z3c.form.widget import FieldWidget
from z3c.form import widget as z3c_widget
from zope import schema
from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile


FIELD_NAME = "categorization_ia_link"


class HtmlSnippetWidget(z3c_widget.Widget):
    template = ViewPageTemplateFile("html_snippet_widget.pt")
    x_imio_application = APPLICATION_ID
    x_imio_municipality = PROJECT_ID

    def update(self):
        # ++add++ : context = container ; edit : context = object
        base = self.context.absolute_url()
        self.endpoint = f"{base}/@@ProcessCategorizeContent"
        self.wid = getattr(self, "name", "categorization_ia_link")
        self.klass = getattr(self, "klass", "")
        self.is_disabled = False

    def render(self):
        return self.template()


class PageAddForm(CustomAddForm):
    portal_type = "imio.smartweb.Page"

    def update(self):
        super(PageAddForm, self).update()

        for group in self.groups:
            if getattr(group, "__name__", "") == "layout":
                if "hide_title" in group.widgets:
                    group.widgets["hide_title"].mode = HIDDEN_MODE
                    group.widgets["hide_title"].value = ["selected"]

        # 2) Inject HTML button in "categorization" group
        if not getattr(self, "groups", None):
            self.updateGroups()

        cat = next(
            (g for g in self.groups if getattr(g, "__name__", "") == "categorization"),
            None,
        )
        if not cat or not getattr(cat, "widgets", None):
            return

        # Avoid doublons (refresh)
        for k in getattr(cat.widgets, "keys", lambda: [])():
            if k.endswith(FIELD_NAME) or k == FIELD_NAME:
                return

        # Create dummy field + FieldWidget(HtmlSnippetWidget)
        zfield = schema.Text(__name__=FIELD_NAME, title="", description="")
        w = FieldWidget(zfield, HtmlSnippetWidget(self.request))
        w.mode = DISPLAY_MODE
        w.context = self.context  # conteneur du futur objet
        w.form = self
        w.ignoreContext = True
        w.label = ""
        w.update()

        try:
            cat.widgets[f"form.widgets.{FIELD_NAME}"] = w
        except Exception:
            try:
                cat.widgets[FIELD_NAME] = w
            except Exception:
                # Fallback bas niveau si nécessaire
                if hasattr(cat.widgets, "_data_keys") and hasattr(
                    cat.widgets, "_widgets"
                ):
                    cat.widgets._data_keys.append(FIELD_NAME)
                    cat.widgets._widgets.append(w)


class PagesAddView(DefaultAddView):
    form = PageAddForm



from Acquisition import aq_inner
from BTrees.OOBTree import OOBTree
from imio.smartweb.core.contents.rest.directory.endpoint import DirectoryEndpointGet
from imio.smartweb.core.contents.rest.events.endpoint import EventsEndpointGet
from imio.smartweb.core.contents.rest.news.endpoint import NewsEndpointGet
from imio.smartweb.core.interfaces import IImioSmartwebCoreLayer
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone.app.layout.navigation.navtree import buildFolderTree
from plone.app.layout.sitemap.sitemap import SiteMapView
from plone.base.interfaces import IPloneSiteRoot
from plone.memoize import ram
from Products.CMFPlone.browser.navigation import CatalogSiteMap as BaseCatalogSiteMap
from Products.CMFPlone.browser.navtree import (
    SitemapNavtreeStrategy as BaseSitemapNavtreeStrategy,
)
from Products.CMFPlone.browser.navtree import (
    SitemapQueryBuilder as BaseSitemapQueryBuilder,
)
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import normalizeString
from zope.component import getMultiAdapter
from zope.interface import implementer
from zope.interface import Interface

import logging
import Missing
import time

logger = logging.getLogger("imio.smartweb.core")


FRIENDLY_TYPES = [
    "Collection",
    "Image",
    "Link",
    "imio.smartweb.Folder",
    "imio.smartweb.Page",
    "imio.smartweb.PortalPage",
    "imio.smartweb.Procedure",
    "imio.smartweb.CirkwiView",
    "imio.smartweb.DirectoryView",
    "imio.smartweb.EventsView",
    "imio.smartweb.NewsView",
]


def cache_key(method, obj, request):
    """We cache data from authentic sources for the sitemap (.xml.gz) for 2 hours."""
    b_start = request.form.get("b_start", "0")
    return f"sitemap_{obj.UID()}_{b_start}_{int(time.time() // 7200)}"


@ram.cache(cache_key)
def get_endpoint_data(obj, request):
    endpoint_mapping = {
        "imio.smartweb.DirectoryView": DirectoryEndpointGet,
        "imio.smartweb.EventsView": EventsEndpointGet,
        "imio.smartweb.NewsView": NewsEndpointGet,
    }
    endpoint_class = endpoint_mapping.get(obj.portal_type)
    if not endpoint_class:
        return {}

    endpoint = endpoint_class()
    if not request.form.get("b_size", 0):
        batch_size = 1000 if obj.portal_type == "imio.smartweb.DirectoryView" else 365
    else:
        batch_size = int(request.form.get("b_size", 0))
    return (
        endpoint.reply_for_given_object(
            obj, request, fullobjects=0, batch_size=batch_size
        )
        or {}
    )


def format_sitemap_items(items, base_url):
    """Format items for sitemap(.xml.gz)"""
    formatted_items = []
    latest_lastmod = None
    item_type = ""
    for item in items:
        item_type = item.get("@type", "")
        lastmod = item.get("lastmod")
        item_id = normalizeString(item.get("title"))
        item_uid = item.get("UID")
        lastmod = item.get("modified") or "1970-01-01T00:00:00Z"
        formatted_items.append(
            {
                "loc": f"{base_url}/{item_id}?u={item_uid}",
                "lastmod": lastmod,
                "Title": item.get("title"),
                "Description": item.get("description", ""),
                "getURL": f"{base_url}/{item_id}?u={item_uid}",
                "getRemoteUrl": Missing.Value,
                "currentItem": False,
                "currentParent": False,
                "normalized_review_state": "published",
                "normalized_portal_type": "imio-smartweb-authsources-item",
            }
        )
        if latest_lastmod is None or lastmod > latest_lastmod:
            latest_lastmod = lastmod
    seo_title = ""
    if item_type == "imio.news.NewsItem":
        seo_title = _("News : SEO Links")
    elif item_type == "imio.directory.Contact":
        seo_title = _("Directory : SEO Links")
    else:
        seo_title = _("Agenda : SEO Links")
    formatted_items.append(
        {
            "loc": f"{base_url}/seo_html",
            "lastmod": latest_lastmod,
            "Title": seo_title,
            "Description": seo_title,
            "getURL": f"{base_url}/seo_html",
            "getRemoteUrl": Missing.Value,
            "currentItem": False,
            "currentParent": False,
            "normalized_review_state": "published",
            "normalized_portal_type": "imio-smartweb-authsources-root",
        }
    )
    return formatted_items


class CustomSiteMapView(SiteMapView):
    """Custom sitemap view. (get items for sitemap.xml.gz)"""

    def objects(self):
        """Returns the data to create the sitemap."""
        friendlytypes = FRIENDLY_TYPES

        catalog = getToolByName(self.context, "portal_catalog")
        query = {
            "portal_type": getToolByName(
                self.context, "plone_utils"
            ).getUserFriendlyTypes(friendlytypes)
        }

        is_plone_site_root = IPloneSiteRoot.providedBy(self.context)
        if not is_plone_site_root:
            query["path"] = "/".join(self.context.getPhysicalPath())

        query["is_default_page"] = True
        default_page_modified = OOBTree()
        for item in catalog.searchResults(query):
            key = item.getURL().rsplit("/", 1)[0]
            value = (item.modified.micros(), item.modified.ISO8601())
            default_page_modified[key] = value

        if is_plone_site_root:
            loc = self.context.absolute_url()
            date = self.context.modified()
            modified = max(
                (date.micros(), date.ISO8601()), default_page_modified.get(loc, (0, ""))
            )
            yield {"loc": loc, "lastmod": modified[1]}

        query["is_default_page"] = False
        for item in catalog.searchResults(query):
            loc = item.getURL()
            date = item.modified
            modified = max(
                (date.micros(), date.ISO8601()), default_page_modified.get(loc, (0, ""))
            )
            yield {"loc": loc, "lastmod": modified[1]}

        brains = catalog(
            portal_type=[
                "imio.smartweb.EventsView",
                "imio.smartweb.NewsView",
                "imio.smartweb.DirectoryView",
            ]
        )
        for brain in brains:
            obj = brain.getObject()
            data = get_endpoint_data(obj, obj.REQUEST)
            yield from format_sitemap_items(data.get("items", {}), obj.absolute_url())


@implementer(IImioSmartwebCoreLayer)
class CatalogSiteMap(BaseCatalogSiteMap):
    def siteMap(self):
        context = aq_inner(self.context)

        queryBuilder = SitemapQueryBuilder(context)
        query = queryBuilder()
        strategy = getMultiAdapter((context, self), ISmartwebNavtreeStrategy)
        base_folder_tree = buildFolderTree(
            context, obj=context, query=query, strategy=strategy
        )

        for child in base_folder_tree.get("children"):
            obj = child.get("item").getObject()
            data = get_endpoint_data(obj, obj.REQUEST)
            if not data:
                continue
            child["children"] = format_sitemap_items(
                data.get("items", []), obj.absolute_url()
            )

        return base_folder_tree


class SitemapQueryBuilder(BaseSitemapQueryBuilder):
    """Builds the request for the sitemap with custom content types."""

    def __init__(self, context):
        self.context = context

    def __call__(self):
        query = {}
        custom_query = getattr(self.context, "getCustomNavQuery", None)
        if custom_query and callable(custom_query):
            query = custom_query()
        query["portal_type"] = FRIENDLY_TYPES
        query["review_state"] = "published"
        return query


class ISmartwebNavtreeStrategy(Interface):
    """Marker interface for the smartweb navtree strategy."""


@implementer(ISmartwebNavtreeStrategy)
class SitemapNavtreeStrategy(BaseSitemapNavtreeStrategy):

    def nodeFilter(self, node):
        # Return all nodes in sitemap even if they are exclude from nav
        return True



# -*- coding: utf-8 -*-

from collective.z3cform.datagridfield.datagridfield import DataGridFieldFactory
from collective.z3cform.datagridfield.registry import DictRow
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.app.registry.browser.controlpanel import ControlPanelFormWrapper
from plone.app.registry.browser.controlpanel import RegistryEditForm
from plone.autoform.directives import omitted
from plone.autoform.directives import mode
from plone.autoform.directives import widget
from plone.registry.interfaces import IRegistry
from plone.z3cform import layout
from Products.statusmessages.interfaces import IStatusMessage

from z3c.form.interfaces import ActionExecutionError
from zope import schema
from zope.component import getUtility
from zope.interface import Interface
from zope.interface import Invalid
from zope.schema import ValidationError
from zope import schema

import json
import logging

logger = logging.getLogger("imio.smartweb.core.browser.controlpanel_siteadmin")

MAX_LENGTH = 33


class LabelTooLong(ValidationError):
    __doc__ = f"Le texte ne peut pas dépasser {MAX_LENGTH} caractères."


def max_length_constraint(value):
    if len(value) > MAX_LENGTH:
        raise LabelTooLong(value)
    return True


class IProcedureTextRowSchema(Interface):

    omitted("label_id")
    mode(label_id="hidden")
    label_id = schema.TextLine(
        title=_("ID"),
        description=_("Unique identifier for the procedure button text"),
        required=False,
    )
    label_fr = schema.TextLine(
        title=_("Label (fr)"),
        description=_("Your button title (french)"),
        constraint=max_length_constraint,
        required=True,
    )
    label_nl = schema.TextLine(
        title=_("Label (nl)"),
        description=_("Your button title (dutch)"),
        constraint=max_length_constraint,
        required=False,
    )
    label_de = schema.TextLine(
        title=_("Label (de)"),
        description=_("Your button title (german)"),
        constraint=max_length_constraint,
        required=False,
    )
    label_en = schema.TextLine(
        title=_("Label (en)"),
        description=_("Your button title (english)"),
        constraint=max_length_constraint,
        required=False,
    )


class ISmartwebSiteAdminControlPanel(Interface):

    widget(procedure_button_text=DataGridFieldFactory)
    procedure_button_text = schema.List(
        title=_("Procedure : Define button text"),
        description=_("Choose procedure submission button text"),
        value_type=DictRow(
            title="Labels",
            schema=IProcedureTextRowSchema,
        ),
        default=[],
        required=False,
    )


class SmartwebSiteAdminControlPanelForm(RegistryEditForm):
    schema = ISmartwebSiteAdminControlPanel
    schema_prefix = "smartweb"
    label = _("Smartweb Site admin Settings")
    # fields = field.Fields(ISmartwebSiteAdminControlPanel)


class SmartwebSiteAdminControlPanelForm(RegistryEditForm):
    schema = ISmartwebSiteAdminControlPanel
    schema_prefix = "smartweb"
    label = _("Smartweb Site admin Settings")

    def applyChanges(self, data):
        rows = data.get("procedure_button_text") or []
        for row in rows:
            all_label_ids = [row.get("label_id") for row in rows if row.get("label_id")]
            numbers = [
                int(label.split("-")[1])
                for label in all_label_ids
                if isinstance(label, str)
                and label.startswith("label-")
                and label.split("-")[1].isdigit()
            ]
            max_number = max(numbers, default=0)
            if row.get("label_id") is None:
                max_number += 1
                row["label_id"] = f"label-{max_number}"
        new_ids = {row.get("label_id") for row in rows if row.get("label_id")}

        registry = getUtility(IRegistry)
        old_rows = registry.get("smartweb.procedure_button_text") or []
        old_ids = {row.get("label_id") for row in old_rows if row.get("label_id")}

        removed_ids = old_ids - new_ids
        if removed_ids:
            brains = api.content.find(portal_type="imio.smartweb.Procedure")
            for removed_id in removed_ids:
                for brain in brains:
                    obj = brain.getObject()
                    if obj.button_ts_label == removed_id:
                        IStatusMessage(self.request).addStatusMessage(
                            f"Label ID '{removed_id}' is still used in Procedure '{obj.absolute_url()}' and will not be removed.",
                            type="error",
                        )
                        return False
        return super().applyChanges(data)


SmartwebSiteAdminControlPanelView = layout.wrap_form(
    SmartwebSiteAdminControlPanelForm, ControlPanelFormWrapper
)



# -*- coding: utf-8 -*-

from imio.smartweb.common.interfaces import ICropping
from imio.smartweb.core.contents import IFolder
from imio.smartweb.core.testing import IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING
from imio.smartweb.core.testing import ImioSmartwebTestCase
from imio.smartweb.core.tests.utils import get_sections_types
from imio.smartweb.core.tests.utils import make_named_image
from plone import api
from plone.app.contenttypes.behaviors.leadimage import ILeadImageBehavior
from plone.app.dexterity.behaviors.metadata import IBasic
from plone.app.imagecropping import PAI_STORAGE_KEY
from plone.app.imagecropping.interfaces import IImageCroppingUtils
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.app.testing import setRoles
from plone.namedfile.file import NamedBlobImage
from plone.testing.zope import Browser
from unittest.mock import patch
from urllib.error import HTTPError
from zope.annotation.interfaces import IAnnotations
from zope.component import getMultiAdapter
from zope.lifecycleevent import Attributes
from zope.lifecycleevent import modified

import transaction


class TestCropping(ImioSmartwebTestCase):
    layer = IMIO_SMARTWEB_CORE_FUNCTIONAL_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.folder = api.content.create(
            container=self.portal,
            type="imio.smartweb.Folder",
            title="Folder",
            id="folder",
        )
        self.page = api.content.create(
            container=self.folder,
            type="imio.smartweb.Page",
            title="Default Page",
            id="defaultpage",
        )

    def test_cropping_adapter(self):
        # footer cropping
        view = getMultiAdapter((self.portal, self.request), name="footer_settings")
        view.add_footer()
        footer = self.portal.listFolderContents(
            contentFilter={"portal_type": "imio.smartweb.Footer"}
        )[0]
        adapter = ICropping(footer, alternate=None)
        self.assertEqual([], adapter.get_scales("background_image", self.request))

        # minisite cropping
        view = getMultiAdapter((self.folder, self.request), name="minisite_settings")
        view.enable()
        adapter = ICropping(self.folder, alternate=None)
        self.assertEqual(
            ["portrait_affiche", "paysage_affiche", "carre_affiche"],
            adapter.get_scales("image", self.request),
        )
        self.assertEqual([], adapter.get_scales("banner", self.request))
        self.assertEqual(["preview"], adapter.get_scales("logo", self.request))
        view.disable()

        # subsite cropping
        view = getMultiAdapter((self.folder, self.request), name="subsite_settings")
        view.enable()
        adapter = ICropping(self.folder, alternate=None)
        self.assertEqual(
            ["portrait_affiche", "paysage_affiche", "carre_affiche"],
            adapter.get_scales("image", self.request),
        )
        self.assertEqual([], adapter.get_scales("banner", self.request))
        self.assertEqual(["preview"], adapter.get_scales("logo", self.request))
        view.disable()

        # folder cropping
        adapter = ICropping(self.folder, alternate=None)
        self.assertIsNotNone(adapter)
        self.assertEqual([], adapter.get_scales("banner", self.request))
        self.assertNotIn("banner", adapter.get_scales("image", self.request))

        # page cropping
        adapter = ICropping(self.page, alternate=None)
        self.assertEqual(
            ["portrait_affiche", "paysage_affiche", "carre_affiche"],
            adapter.get_scales("image", self.request),
        )

        # sections cropping
        section_types = get_sections_types()
        for section_type in section_types:
            section = api.content.create(
                container=self.page,
                type=section_type,
                title="Title of my {}".format(section_type),
            )
            adapter = ICropping(section, alternate=None)
            self.assertIsNotNone(adapter)
            self.assertEqual([], adapter.get_scales("background_image", self.request))
            self.assertNotIn("banner", adapter.get_scales("image", self.request))

    def test_uncroppable_fields(self):
        self.folder.banner = NamedBlobImage(**make_named_image("plone.png"))
        self.folder.image = NamedBlobImage(**make_named_image("plone.png"))
        adapter = IImageCroppingUtils(self.folder, alternate=None)
        self.assertIsNotNone(adapter)
        self.assertEqual(len(list(adapter._image_field_values())), 1)
        self.assertEqual(adapter.image_field_names(), ["image"])

    def test_cropping_view(self):
        self.folder.banner = NamedBlobImage(**make_named_image("plone.png"))
        self.folder.image = NamedBlobImage(**make_named_image("plone.png"))
        cropping_view = getMultiAdapter(
            (self.folder, self.request), name="croppingeditor"
        )
        self.assertEqual(len(list(cropping_view._scales("banner"))), 0)
        self.assertEqual(len(list(cropping_view._scales("image"))), 3)
        self.assertNotIn("Banner", cropping_view())
        self.assertIn("Lead Image", cropping_view())

    def test_removing_old_cropping(self):
        self.folder.banner = NamedBlobImage(**make_named_image("plone.png"))
        self.folder.image = NamedBlobImage(**make_named_image("plone.png"))

        modified(self.folder, Attributes(IBasic, "IBasic.title"))
        annotation = IAnnotations(self.folder).get(PAI_STORAGE_KEY)
        self.assertEqual(annotation, None)

        modified(
            self.folder, Attributes(ILeadImageBehavior, "ILeadImageBehavior.image")
        )
        annotation = IAnnotations(self.folder).get(PAI_STORAGE_KEY)
        self.assertEqual(annotation, None)

        view = self.folder.restrictedTraverse("@@crop-image")
        view._crop(fieldname="image", scale="portrait_affiche", box=(1, 1, 200, 200))
        annotation = IAnnotations(self.folder).get(PAI_STORAGE_KEY)
        self.assertEqual(annotation, {"image_portrait_affiche": (1, 1, 200, 200)})

        modified(self.folder, Attributes(IFolder, "banner"))
        annotation = IAnnotations(self.folder).get(PAI_STORAGE_KEY)
        self.assertEqual(annotation, {"image_portrait_affiche": (1, 1, 200, 200)})

        modified(
            self.folder, Attributes(ILeadImageBehavior, "ILeadImageBehavior.image")
        )
        annotation = IAnnotations(self.folder).get(PAI_STORAGE_KEY)
        self.assertEqual(annotation, {})

    def test_removing_image_and_scales(self):
        self.folder.image = NamedBlobImage(**make_named_image("plone.png"))
        modified(
            self.folder, Attributes(ILeadImageBehavior, "ILeadImageBehavior.image")
        )
        scales = self.folder.restrictedTraverse("@@images")
        image = scales.scale("image", scale="paysage_vignette")
        image_url = image.url
        transaction.commit()
        browser = Browser(self.layer["app"])
        browser.addHeader(
            "Authorization",
            "Basic %s:%s"
            % (
                TEST_USER_NAME,
                TEST_USER_PASSWORD,
            ),
        )
        browser.open(image_url)
        content = browser.contents
        # Got picture.
        self.assertIsInstance(content, bytes)

        # Old situation before fix
        # Ensure browser still can access the image URL
        # Situation BEFORE we make clear_image_scales function
        self.folder.image = None  # remive image from object
        with patch("imio.smartweb.core.subscribers.clear_image_scales") as mock_clear:
            modified(
                self.folder, Attributes(ILeadImageBehavior, "ILeadImageBehavior.image")
            )
            transaction.commit()
            browser.open(image_url)
            content = browser.contents
            self.assertIsInstance(content, bytes)

        # New situation with fix
        # Call subrscriber and pass in "real" clear_image_scales function
        modified(
            self.folder, Attributes(ILeadImageBehavior, "ILeadImageBehavior.image")
        )
        transaction.commit()
        with self.assertRaises(HTTPError) as context:
            browser.open(image_url)
            self.assertIn("NotFound", str(context.exception))



# -*- coding: utf-8 -*-
from imio.smartweb.core.contents.rest.search.endpoint import get_default_view_url
from imio.smartweb.core.utils import get_scale_url
from imio.smartweb.core.behaviors.minisite import IImioSmartwebMinisite
from imio.smartweb.core.behaviors.subsite import IImioSmartwebSubsite
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.app.layout.viewlets import common
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot
from Products.CMFPlone.utils import parent
from zope.component import getMultiAdapter


class BaseFooterViewlet(common.ViewletBase):
    _footer = None

    def available(self):
        return self.footer is not None

    def background_style(self):
        footer = self.footer
        if not footer.background_image:
            return ""
        scale_url = get_scale_url(
            self.footer, self.request, "background_image", "affiche", "paysage"
        )
        css_bg_image = f"background-image:url({scale_url});"
        css_bg_size = "background-size:cover;"
        return " ".join([css_bg_image, css_bg_size])

    def update(self):
        context_state = getMultiAdapter(
            (self.context, self.request), name="plone_context_state"
        )
        if not self.available():
            return
        self.sections = self.footer.listFolderContents()
        self.current_page_url = context_state.current_page_url

    def seo_auth_sources_links(self):
        view_name = "seo_html"
        news_link = get_default_view_url("news")
        news_link = f"{news_link}/{view_name}"
        events_link = get_default_view_url("events")
        events_link = f"{events_link}/{view_name}"
        directory_link = get_default_view_url("directory")
        directory_link = f"{directory_link}/{view_name}"
        return [
            {"label": _("Access to news links"), "url": news_link},
            {"label": _("Access to events links"), "url": events_link},
            {"label": _("Access to directory links"), "url": directory_link},
        ]


class FooterViewlet(BaseFooterViewlet):
    css_id = "smartweb-footer"

    @property
    def footer(self):
        if self._footer is not None:
            return self._footer
        root = api.portal.get_navigation_root(self.context)
        if IImioSmartwebMinisite.providedBy(root):
            # don't display portal footer in a minisite
            return
        available_langs = api.portal.get_registry_record("plone.available_languages")
        if IPloneSiteRoot.providedBy(root) and len(available_langs) > 1:
            # multilingual : we should display current lang footer
            current_lang = api.portal.get_current_language()[:2]
            root = getattr(root, current_lang, root)
        footers = root.listFolderContents(
            contentFilter={"portal_type": "imio.smartweb.Footer"}
        )
        if len(footers) > 0:
            self._footer = footers[0]
            return self._footer


class SubsiteFooterViewlet(BaseFooterViewlet):
    css_id = "smartweb-subsite-footer"

    @property
    def footer(self):
        if self._footer is not None:
            return self._footer
        obj = self.context
        while not INavigationRoot.providedBy(obj):
            if IImioSmartwebSubsite.providedBy(obj):
                footers = obj.listFolderContents(
                    contentFilter={"portal_type": "imio.smartweb.Footer"}
                )
                if len(footers) > 0:
                    self._footer = footers[0]
                    return self._footer
            obj = parent(obj)


class MinisiteFooterViewlet(BaseFooterViewlet):
    css_id = "smartweb-minisite-footer"

    @property
    def footer(self):
        if self._footer is not None:
            return self._footer
        root = api.portal.get_navigation_root(self.context)
        if IImioSmartwebMinisite.providedBy(root):
            footers = root.listFolderContents(
                contentFilter={"portal_type": "imio.smartweb.Footer"}
            )
            if len(footers) > 0:
                self._footer = footers[0]
                return self._footer



from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.app.z3cform.interfaces import IPloneFormLayer
from z3c.form.browser.text import TextWidget
from z3c.form.interfaces import IFieldWidget
from z3c.form.interfaces import IFormLayer
from z3c.form.interfaces import IWidget
from z3c.form.widget import FieldWidget
from zope.component import adapter
from zope.i18n import translate
from zope.interface import implementer
from zope.interface import implementer_only
from zope.schema.interfaces import ITextLine


class ISuggestedIATitlesWidget(IWidget):
    pass


@implementer_only(ISuggestedIATitlesWidget)
class SuggestedIATitlesWidget(TextWidget):
    klass = "text-widget form-control suggestedtitles-with-button"

    def update(self):
        super().update()
        self.pattern_options = {
            "sourceSelector": "#form-widgets-title",
            "buttonLabel": "Get titles",
        }

    @property
    def data_pat_plone_modal(self):
        language = api.portal.get_current_language(context=self.context)
        title = translate(_("Title suggestions"), target_language=language)
        return f"title: {title}; width: 600; loadLinksWithinModal: false"


@implementer(IFieldWidget)
@adapter(ITextLine, IPloneFormLayer)
def SuggestedIATitlesFieldWidget(field, request) -> IFieldWidget:
    return FieldWidget(field, SuggestedIATitlesWidget(request))



# -*- coding: utf-8 -*-

from collective.anysurfer.interfaces import ILayerSpecific
from collective.messagesviewlet.interfaces import ICollectiveMessagesviewletLayer
from collective.solr.browser.interfaces import IThemeSpecific
from imio.smartweb.common.interfaces import IImioSmartwebCommonLayer
from plone.app.contenttypes.interfaces import IPloneAppContenttypesLayer
from Products.CMFPlone.browser.interfaces import ISiteMap
from zope.interface import Interface
from imio.gdpr.interfaces import IImioGdprLayer


class ISmartwebIcon(Interface):
    """Marker interface to easily find all registered icons in registry."""


class IViewWithoutLeadImage(Interface):
    """Marker interface for views that shouldn't show leadimage."""


class IImioSmartwebCoreLayer(
    IImioGdprLayer,
    IImioSmartwebCommonLayer,
    IPloneAppContenttypesLayer,
    ILayerSpecific,
    IThemeSpecific,
    ICollectiveMessagesviewletLayer,
    ISiteMap,
):
    """Marker interface that defines a browser layer."""


class IImioIdeaboxLayer(Interface):
    """Marker interface that defines a browser layer."""


class IArcgisViewUtils(Interface):
    """ """

    def get_portal_item_id():
        """Return id of a map"""


class IOgpViewUtils(Interface):
    """ """


class IOdwbViewUtils(Interface):
    """ """

    def is_odwb_widget_plugins():
        """Return True if we are in the generic odwb widget plugins"""


class IHtmxViewUtils(Interface):
    """ """


class INoIndexedUtils(Interface):
    """ """



# -*- coding: utf-8 -*-
from datetime import datetime
from imio.smartweb.core.utils import get_json
from imio.smartweb.locales import SmartwebMessageFactory as _
from imio.smartweb.core.contents.rest.view import BaseRestView
from imio.smartweb.core.interfaces import IOgpViewUtils
from plone import api
from zope.interface import implementer


@implementer(IOgpViewUtils)
class EventsViewView(BaseRestView):
    """EventsView view"""

    @property
    def propose_url(self):
        return api.portal.get_registry_record("smartweb.propose_events_url")

    @property
    def display_map(self):
        return self.context.display_map

    @property
    def only_past_events(self):
        return self.context.only_past_events

    @property
    def display_agendas_titles(self):
        return self.context.display_agendas_titles

    @property
    def show_categories_or_topics(self):
        return self.context.show_categories_or_topics

    @property
    def event(self):
        event = self._formated_event(self.item)
        return event

    def _formated_event(self, data):
        current_lang = api.portal.get_current_language()[:2]

        # Récupération des données de base
        name = data.get(f"title_{current_lang}") or data.get("title")
        text = data.get(f"text_{current_lang}") or data.get("text")
        description = data.get(f"description_{current_lang}") or data.get("description")
        number = data.get("number")
        street = data.get("street")
        city = data.get("city")
        zipcode = data.get("zipcode")
        country = data.get("country").get("title") if data.get("country") else None
        full_address = self._format_address(street, number, zipcode, city, country)
        phone = data.get("contact_phone")
        email = data.get("contact_email")
        event_url = data.get("event_url")
        event_type = (
            data.get("event_type", {}).get("title", "")
            if data.get("event_type")
            else None
        )

        # Dates management
        start_date = data.get("start")
        end_date = data.get("end")
        start_date_obj = datetime.fromisoformat(start_date)
        end_date_obj = datetime.fromisoformat(end_date)
        whole_day = data.get("whole_day", False)
        if whole_day:
            formatted_date = start_date_obj.date().strftime("%d/%m/%Y")
            wd = _(f"Whole day")
            display_period_text = _(f"{formatted_date} ({wd})")
        else:
            formatted_start = start_date_obj.strftime("%d/%m/%Y %H:%M")
            formatted_end = end_date_obj.strftime("%d/%m/%Y %H:%M")
            display_period_text = f"{formatted_start} - {formatted_end}"

        # Geolocation
        geo = data.get("geolocation") or {}
        latitude, longitude = geo.get("latitude"), geo.get("longitude")

        if not latitude or not longitude:
            geo = None
            latitude = longitude = None

        # Picture
        image_info = data.get("image") or {}
        image_url = None

        # Choisir dynamiquement la scale 'portrait_affiche' si disponible
        if "scales" in image_info and "portrait_affiche" in image_info["scales"]:
            image_url = image_info["scales"]["portrait_affiche"]["download"]
        elif "download" in image_info:
            # Fallback sur l'image originale
            image_url = image_info["download"]

        if not data:
            return None
        # Construction du JSON simplifié
        prefix_address = _("Address")
        prefix_email = _("Email")
        prefix_description = _("Description")
        prefix_category = _("Event category")
        prefix_type = _("Event type")
        event = {
            "name": name,
            "text": text,
            "address": f"{prefix_address}: {full_address}" if full_address else None,
            "phone": phone,
            "email": f"{prefix_email}: {email}" if email else None,
            "description": (
                f"{prefix_description}: {description}" if description else None
            ),
            "url": event_url,
            "geolocation": (latitude, longitude) if geo else None,
            "event_type": f"{prefix_type}: {event_type}" if event_type else None,
            "image_url": image_url if image_url else None,
            "period_text": display_period_text,
            "start_date": start_date_obj.strftime("%d/%m/%Y %H:%M"),
            "end_date": end_date_obj.strftime("%d/%m/%Y %H:%M"),
        }
        return event



from imio.smartweb.common.utils import get_vocabulary
from imio.smartweb.common.browser.ia import BaseIAView
from imio.smartweb.common.config import IPA_URL
from imio.smartweb.common.utils import get_vocabulary
from imio.smartweb.core.contents import IPages
from imio.smartweb.core.contents import ISectionText
from imio.smartweb.core.utils import get_categories
from plone import api
from Products.Five import BrowserView
from zope.i18n import translate

import json
import os
import requests


class ProcessSuggestedTitlesView(BaseIAView):

    def __call__(self):
        self.request.response.setHeader(
            "Content-Type", "application/json; charset=utf-8"
        )
        current_html = self.request.form.get("text", "")
        payload = {
            "input": current_html,
            "expansion_target": 50,
        }
        url = f"{IPA_URL}/suggest-titles"
        response = requests.post(url, headers=self.headers, json=payload)
        if response.status_code != 200:
            return current_html
        data = response.json()
        if not data:
            return current_html
        return json.dumps(data)


class ProcessCategorizeContentView(BaseIAView):

    def __init__(self, context, request):
        super().__init__(context, request)
        self.current_lang = api.portal.get_current_language()[:2]

    def _get_structured_data_from_vocabulary(self, vocabulary_name):
        voc = get_vocabulary(vocabulary_name)
        voc_translated_dict = [
            {
                "title": translate(t.title, target_language=self.current_lang),
                "token": t.token,
            }
            for t in voc
        ]
        return voc_translated_dict

    def _ask_categorization_to_ia(self, text, voc):
        payload = {"input": text, "vocabulary": voc, "unique": False}
        url = f"{IPA_URL}/categorize-content"
        response = requests.post(url, headers=self.headers, json=payload)
        data = response.json()
        return data

    def __call__(self):
        results = {}

        self.request.response.setHeader(
            "Content-Type", "application/json; charset=utf-8"
        )
        all_text = ""
        if IPages.providedBy(self.context):
            for item in self.context.objectItems():
                obj = item[1]
                if ISectionText.providedBy(obj):
                    all_text += getattr(obj.text, "output", "")

        # If agent doesn't register prevsouly a categorization so IA can bring categorization
        if self.context.taxonomy_page_category is None:
            categories_taxo = get_categories()
            categories_voca = categories_taxo.makeVocabulary(self.current_lang).inv_data
            categories_voc = [
                {"title": v, "token": k} for k, v in categories_voca.items()
            ]
            data = self._ask_categorization_to_ia(all_text, categories_voc)
            if not data:
                return ""
            results["form-widgets-page_category-taxonomy_page_category"] = [
                {"title": r.get("title"), "token": r.get("token")}
                for r in data.get("result")
            ]

        iam_voc = self._get_structured_data_from_vocabulary(
            "imio.smartweb.vocabulary.IAm"
        )
        data = self._ask_categorization_to_ia(all_text, iam_voc)
        if not data:
            return ""
        # iam tokens already register in field
        iam_tokens = self.context.iam or []
        ia_iam = [
            {"title": r.get("title"), "token": r.get("token")}
            for r in data.get("result")
        ]
        results["form-widgets-IAm-iam"] = ia_iam
        for iam_token in iam_tokens:
            if iam_token not in (t.get("token") for t in ia_iam):
                iam = next((t for t in topics_voc if t.get("token") == iam_token), [])
                results["form-widgets-IAm-iam"].append(iam)

        topics_voc = self._get_structured_data_from_vocabulary(
            "imio.smartweb.vocabulary.Topics"
        )
        data = self._ask_categorization_to_ia(all_text, topics_voc)
        if not data:
            return ""
        # Topic tokens already register in field
        topic_tokens = self.context.topics or []
        ia_topics = [
            {"title": r.get("title"), "token": r.get("token")}
            for r in data.get("result")
        ]

        results["form-widgets-ITopics-topics"] = ia_topics

        for topic_token in topic_tokens:
            if topic_token not in (t.get("token") for t in ia_topics):
                topic = next(
                    (t for t in topics_voc if t.get("token") == topic_token), []
                )
                results["form-widgets-ITopics-topics"].append(topic)

        return json.dumps(
            {
                "ok": True,
                "message": "Catégorisation calculée",
                "data": results,
            }
        )



# -*- coding: utf-8 -*-

from imio.smartweb.core.behaviors.subsite import IImioSmartwebSubsite
from imio.smartweb.core.contents import IFolder
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from plone.app.layout.navigation.interfaces import INavigationRoot
from Products.CMFPlone.interfaces.constrains import DISABLED
from Products.CMFPlone.interfaces.constrains import ISelectableConstrainTypes
from Products.Five.browser import BrowserView
from plone.protect.interfaces import IDisableCSRFProtection
from zope.interface import alsoProvides


class CreateContentsForMigration(BrowserView):

    def __call__(self):
        # Faut lancer la migration depuis /Plone
        alsoProvides(self.request, IDisableCSRFProtection)
        self.create_content("actualites", "Actualités")
        self.create_content("images", "Images")

    def create_content(self, id, title):
        fr = [obj for obj in self.context.items() if obj[0] == "fr"][0][1]
        if id not in [i[0] for i in fr.items()]:
            api.content.create(
                id=id, title=title, container=fr, type="imio.smartweb.Folder"
            )



# -*- coding: utf-8 -*-

from imio.smartweb.core.contents.rest.view import BaseRestView
from imio.smartweb.core.interfaces import IOgpViewUtils
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from zope.interface import implementer


@implementer(IOgpViewUtils)
class NewsViewView(BaseRestView):
    """NewsView view"""

    @property
    def propose_url(self):
        return api.portal.get_registry_record("smartweb.propose_news_url")

    @property
    def display_newsfolders_titles(self):
        return self.context.display_newsfolders_titles

    @property
    def show_categories_or_topics(self):
        return self.context.show_categories_or_topics

    @property
    def news(self):
        news = self._formated_news(self.item)
        return news

    def _formated_news(self, data):
        current_lang = api.portal.get_current_language()[:2]
        # Récupération des données de base
        name = data.get(f"title_{current_lang}") or data.get("title")
        text = data.get(f"text_{current_lang}") or data.get("text")
        description = data.get(f"description_{current_lang}") or data.get("description")
        phone = data.get("contact_phone")
        email = data.get("contact_email")
        url = data.get("site_url")

        # Picture
        image_info = data.get("image") or {}
        image_url = None

        # Choisir dynamiquement la scale 'portrait_affiche' si disponible
        if "scales" in image_info and "portrait_affiche" in image_info["scales"]:
            image_url = image_info["scales"]["portrait_affiche"]["download"]
        elif "download" in image_info:
            # Fallback sur l'image originale
            image_url = image_info["download"]

        if not data:
            return None
        # Construction du JSON simplifié
        prefix_email = _("Email")
        prefix_description = _("Description")
        news = {
            "name": name,
            "text": text,
            "phone": phone,
            "email": f"{prefix_email}: {email}" if email else None,
            "description": (
                f"{prefix_description}: {description}" if description else None
            ),
            "url": url,
            "image_url": image_url if image_url else None,
        }
        return news



# -*- coding: utf-8 -*-

from plone.app.layout.viewlets import common
from plone.app.layout.viewlets.httpheaders import HeaderViewlet
from zope.component import queryMultiAdapter
from zope.viewlet.interfaces import IViewletManager


class PortalHeaderTopContainerViewlet(common.ViewletBase):
    """Viewlet containing the top header viewlets manager"""


class IPortalHeaderTopContainerViewletManager(IViewletManager):
    """Viewlet manager containing top header viewlets"""


class SeoViewlet(HeaderViewlet):
    """Pages that should not be indexed by search engines"""

    def render(self):
        request = self.request
        b_start = int(request.get("b_start", 0))
        b_size = int(request.get("b_size", 10))
        total = self.view.total if hasattr(self.view, "total") else 0

        base_url = self.context.absolute_url()
        links = []

        if b_start > 0:
            prev_start = max(0, b_start - b_size)
            links.append(
                f'<link rel="prev" total="{total}" href="{base_url}/seo_html?b_start={prev_start}&amp;b_size={b_size}" />'
            )

        if (b_start + b_size) < total:
            next_start = b_start + b_size
            links.append(
                f'<link rel="next" total="{total}" href="{base_url}/seo_html?b_start={next_start}&amp;b_size={b_size}" />'
            )

        return "\n".join(links)

    def update(self):
        super(SeoViewlet, self).update()
        # This page should not be indexed, we set the X-Robots-Tag header
        self.request.response.setHeader("X-Robots-Tag", "noindex, follow")



from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
import pkg_resources


class AccessibilityInfo(BrowserView):
    hide_herobanner = True
    index = ViewPageTemplateFile(
        pkg_resources.resource_filename(
            "Products.CMFPlone", "browser/templates/accessibility-info.pt"
        )
    )

    def __call__(self):
        return self.index()



# -*- coding: utf-8 -*-
# imio/smartweb/core/browser/categorization_button_edit.py
from imio.smartweb.common.config import APPLICATION_ID
from imio.smartweb.common.config import PROJECT_ID
from imio.smartweb.core.browser.forms import SmartwebCustomEditForm
from imio.smartweb.core.contents import ISectionText

from plone.z3cform import layout

from zope import schema
from z3c.form.interfaces import DISPLAY_MODE, HIDDEN_MODE
from z3c.form.widget import FieldWidget
from z3c.form import widget as z3c_widget
from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile


FIELD_NAME = "categorization_ia_link"  # Internal id for dummy field


class HtmlSnippetWidget(z3c_widget.Widget):
    """Widget HTML (bouton + JS) avec template ZPT."""

    template = ViewPageTemplateFile("html_snippet_widget.pt")
    x_imio_application = APPLICATION_ID
    x_imio_municipality = PROJECT_ID

    def update(self):
        # edit : context == objet
        base = self.context.absolute_url()
        self.endpoint = f"{base}/@@ProcessCategorizeContent"
        # Unique id for button + status zone
        self.wid = getattr(self, "name", FIELD_NAME)

        # Désactiver le bouton si aucune section texte avec contenu n'est présente
        self.klass = getattr(self, "klass", "")
        has_text_content = False

        # Vérifie si le contexte contient au moins une section texte avec du contenu
        try:
            for item in getattr(self.context, "objectItems", lambda: [])():
                obj = item[1]
                if ISectionText.providedBy(obj):
                    # Vérifier si la section texte a du contenu (non vide)
                    text_output = getattr(getattr(obj, "text", None), "output", "")
                    if text_output and text_output.strip():
                        has_text_content = True
                        break
        except Exception:
            pass

        if not has_text_content:
            self.klass = f"{self.klass} disabled".strip() if self.klass else "disabled"
            self.is_disabled = True
        else:
            self.is_disabled = False

    def render(self):
        return self.template()


class PageEditForm(SmartwebCustomEditForm):
    """Vue edit custom, avec bouton 'Catégoriser' injecté en haut de 'categorization'."""

    def update(self):
        super(PageEditForm, self).update()

        # 1) Hide hide_title in 'layout' group
        for group in getattr(self, "groups", []):
            if (
                getattr(group, "__name__", "") == "layout"
                and "hide_title" in group.widgets
            ):
                group.widgets["hide_title"].mode = HIDDEN_MODE
                group.widgets["hide_title"].value = ["selected"]

        # 2) Inject button on top of 'categorization' fieldset
        if not getattr(self, "groups", None):
            return

        # Find 'categorization' group
        cat = next(
            (g for g in self.groups if getattr(g, "__name__", "") == "categorization"),
            None,
        )
        if not cat or not getattr(cat, "widgets", None):
            return

        # Avoid doublons (refresh) if already here => stop
        existing_keys = list(getattr(cat.widgets, "keys", lambda: [])())
        for k in existing_keys:
            if (
                k.endswith(FIELD_NAME)
                or k == FIELD_NAME
                or k.endswith(f"form.widgets.{FIELD_NAME}")
            ):
                return

        # Create dummy field + FieldWidget(HtmlSnippetWidget)
        zfield = schema.Text(__name__=FIELD_NAME, title="", description="")
        w = FieldWidget(zfield, HtmlSnippetWidget(self.request))
        w.mode = DISPLAY_MODE
        w.context = self.context
        w.form = self
        w.ignoreContext = True
        w.label = ""
        w.update()  # prépare endpoint/wid

        key = f"form.widgets.{FIELD_NAME}"

        if hasattr(cat.widgets, "_data_keys") and hasattr(cat.widgets, "_widgets"):
            # Remove residual occurrences
            try:
                while key in cat.widgets._data_keys:
                    idx = cat.widgets._data_keys.index(key)
                    cat.widgets._data_keys.pop(idx)
                    cat.widgets._widgets.pop(idx)
            except Exception:
                pass
            cat.widgets._data_keys.insert(0, key)
            cat.widgets._widgets.insert(0, w)
            return

        # fallback : rebuild mapping with our with our widget
        try:
            existing = [(k, cat.widgets[k]) for k in list(cat.widgets.keys())]
            try:
                cat.widgets.clear()
            except Exception:
                pass
            # Firstly, our widget
            cat.widgets[key] = w
            # Next...
            for k, ww in existing:
                if k != key:
                    cat.widgets[k] = ww
        except Exception:
            # Last resort: If nothing else worked
            try:
                cat.widgets[key] = w
            except Exception:
                pass


PageEditView = layout.wrap_form(PageEditForm)
